@@ -39,16 +39,13 @@ Implementation
 '''
 import os
 import sys
-try:
-    import pulsar
-except ImportError:  # pragma nocover
-    sys.path.append('../../')
-    import pulsar
 
+from pulsar import coroutine_return
 from pulsar.apps import ws, wsgi
+from pulsar.utils.log import process_global
 from pulsar.utils.system import json
 try:
-    from .tx import twisted
+    from pulsar.apps.tx import twisted
     from twisted.internet import protocol, endpoints, reactor
     from twisted.mail import imap4
 except ImportError:  # pragma    nocover
@@ -59,26 +56,31 @@ ASSET_DIR = os.path.join(os.path.dirname(__file__), 'assets')
 
 def mail_client(cfg, timeout=10):
     '''Create a new mail client using twisted IMAP4 library.'''
-    endpoint = endpoints.clientFromString(reactor, cfg.mail_incoming)
-    endpoint._timeout = timeout
-    factory = protocol.Factory()
-    factory.protocol = imap4.IMAP4Client
-    client = yield endpoint.connect(factory)
-    yield client.login(cfg.mail_username, cfg.mail_password)
-    yield client.select('INBOX')
-    yield client
-    #info = yield client.fetchEnvelope(imap4.MessageSet(1))
-    #print 'First message subject:', info[1]['ENVELOPE'][1]
+    key = (cfg.mail_incoming, cfg.mail_username, cfg.mail_password)
+    client = process_global(key)
+    if not client:
+        endpoint = endpoints.clientFromString(reactor, cfg.mail_incoming)
+        endpoint._timeout = timeout
+        factory = protocol.Factory()
+        factory.protocol = imap4.IMAP4Client
+        client = yield endpoint.connect(factory)
+        yield client.login(cfg.mail_username, cfg.mail_password)
+        yield client.select('INBOX')
+        process_global(key, client, True)
+    coroutine_return(client)
+    # info = yield client.fetchEnvelope(imap4.MessageSet(1))
+    # print 'First message subject:', info[1]['ENVELOPE'][1]
 
 
 class WsMail(ws.WS):
-    ''':ref:`Websocket handler <websocket-handler>` for fetching and
-sending mail via the twisted IMAP4 library.'''
+    '''A :class:`.WS` handler for fetching and sending mail via the
+    twisted IMAP4 library
+    '''
     def on_open(self, websocket):
         '''When the websocket starts, it create a new mail client.'''
-        request = websocket.request
-        client = yield mail_client(request.cache.cfg)
-        #add the mail client to the environ cache
+        request = websocket.handshake
+        client = yield mail_client(request.cfg)
+        # add the mail client to the environ cache
         request.cache.mailclient = client
         # retrieve the list of mailboxes and them to the client
         yield self._send_mailboxes(websocket)
@@ -93,7 +95,7 @@ sending mail via the twisted IMAP4 library.'''
                 self.write(request, json.dumps({'mailbox': mailbox}))
 
     def _send_mailboxes(self, websocket):
-        request = websocket.request
+        request = websocket.handshake
         result = yield request.cache.mailclient.list("", "*")
         result = sorted([e[2] for e in result])
         websocket.write(json.dumps({'list': result}))
@@ -101,16 +103,17 @@ sending mail via the twisted IMAP4 library.'''
 
 class WebMail(wsgi.LazyWsgi):
 
-    def setup(self):
+    def setup(self, environ):
         return wsgi.WsgiHandler([ws.WebSocket('/message', WsMail()),
                                  wsgi.MediaRouter('/media', ASSET_DIR),
                                  wsgi.Router('/', get=self.home)])
 
     def home(self, request):
         data = open(os.path.join(ASSET_DIR, 'mail.html')).read()
+        response = request.response
         request.response.content = data % request.environ
         request.response.content_type = 'text/html'
-        return request.response.start()
+        return response
 
 
 def server(name='webmail', callable=None, **kwargs):
