@@ -62,8 +62,9 @@ import time
 import pulsar
 from pulsar import command
 from pulsar.utils.config import section_docs
-from pulsar.apps.data import (start_store, create_store,
-                              DEFAULT_PULSAR_STORE_ADDRESS)
+from pulsar.apps.data import create_store
+from pulsar.apps.data.stores import start_store
+from pulsar.apps.ds import DEFAULT_PULSAR_STORE_ADDRESS
 
 from .models import *
 from .backend import *
@@ -107,7 +108,7 @@ class TaskBackendConnection(TaskSetting):
     default = ""
     meta = 'CONNECTION_STRING'
     desc = '''\
-        Connection string for the backend storing :class:`.Task`.
+        Connection string for the backend storing tasks.
 
         If the value is not available (default) it uses as fallback the
         :ref:`data_store <setting-data_store>` value. If still not
