@@ -3,9 +3,7 @@ from pulsar.utils.httpurl import iteritems
 from pulsar.apps.test.result import Plugin
 
 
-__all__ = ['WrapTest',
-           'TestOption',
-           'TestPlugin']
+__all__ = ['WrapTest', 'TestPlugin']
 
 
 def as_test_setting(setting):
@@ -14,12 +12,6 @@ def as_test_setting(setting):
     return setting
 
 
-class TestOption(pulsar.Setting):
-    virtual = True
-    app = 'test'
-    section = "Test"
-
-
 class WrapTest(object):
     '''Wrap an underlying test case'''
     def __init__(self, test):
