@@ -41,10 +41,10 @@ Wsgi Response
 .. _TLS: http://en.wikipedia.org/wiki/Transport_Layer_Security
 '''
 import re
-from functools import reduce
+from functools import reduce, partial
 from io import BytesIO
 
-from pulsar import Future, coroutine_return
+from pulsar import Future, chain_future
 from pulsar.utils.system import json
 from pulsar.utils.multipart import parse_form_data, parse_options_header
 from pulsar.utils.structures import AttributeDictionary
@@ -335,8 +335,9 @@ class EnvironMixin(object):
     @property
     def cache(self):
         '''An :ref:`attribute dictionary <attribute-dictionary>` of
-pulsar-specific data stored in the :attr:`environ` at the wsgi-extension
-key ``pulsar.cache``.'''
+        pulsar-specific data stored in the :attr:`environ` at
+        the wsgi-extension key ``pulsar.cache``
+        '''
         return self.environ['pulsar.cache']
 
     @property
@@ -429,7 +430,7 @@ class WsgiRequest(EnvironMixin):
     @property
     def urlargs(self):
         '''Dictionary of url parameters obtained when matching a
-:ref:`router <wsgi-router>` with this request :attr:`path`.'''
+        :ref:`router <wsgi-router>` with this request :attr:`path`.'''
         return self.cache.urlargs
 
     @property
@@ -470,6 +471,10 @@ class WsgiRequest(EnvironMixin):
         '''
         return self.environ.get('PATH_INFO', '/')
 
+    @property
+    def uri(self):
+        return self.absolute_uri()
+
     @property
     def method(self):
         '''The request method (uppercase).'''
@@ -516,27 +521,29 @@ class WsgiRequest(EnvironMixin):
         '''
         return self.data_and_files(files=False)
 
-    def _data_and_files(self, data=True, files=True):
+    def _data_and_files(self, data=True, files=True, future=None):
         result = {}, None
-        stream = self.environ.get('wsgi.input')
         chunk = None
-        try:
+        if future is None:
+            stream = self.environ.get('wsgi.input')
             if self.method not in ENCODE_URL_METHODS and stream:
                 chunk = stream.read()
                 if isinstance(chunk, Future):
-                    chunk = yield chunk
-                content_type, options = self.content_type_options
-                charset = options.get('charset', 'utf-8')
-                if content_type in JSON_CONTENT_TYPES:
-                    result = json.loads(chunk.decode(charset)), None
-                else:
-                    self.environ['wsgi.input'] = BytesIO(chunk)
-                    result = parse_form_data(self.environ, charset)
-        finally:
-            self.cache.data_and_files = result
-            if chunk is not None:
+                    return chain_future(
+                        chunk, partial(self._data_and_files, data, files))
+        else:
+            chunk = future
+        if chunk is not None:
+            content_type, options = self.content_type_options
+            charset = options.get('charset', 'utf-8')
+            if content_type in JSON_CONTENT_TYPES:
+                result = json.loads(chunk.decode(charset)), None
+            else:
                 self.environ['wsgi.input'] = BytesIO(chunk)
-        coroutine_return(self.data_and_files(data, files))
+                result = parse_form_data(self.environ, charset)
+            self.environ['wsgi.input'] = BytesIO(chunk)
+        self.cache.data_and_files = result
+        return self.data_and_files(data, files)
 
     @cached_property
     def url_data(self):
